package cn.sh.stc.sict.theme.hpgp.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpBusyIdlePredictionMapper;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpDepartmentRankMapper;
import cn.sh.stc.sict.theme.hpgp.model.HpgpBusyIdlePrediction;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hpgp.service.HpgpBusyIdlePredictionService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 黄浦全科导诊——忙闲预测(HpgpBusyIdle_ prediction)表服务实现类
 *
 * @author zhangly
 * @since 2022-01-14 09:56:43
 */
@AllArgsConstructor
@Service("hpgpBusyIdlPredictionService")
public class HpgpBusyIdlePredictionServiceImpl extends ServiceImpl<HpgpBusyIdlePredictionMapper, HpgpBusyIdlePrediction> implements HpgpBusyIdlePredictionService {
    private final HpgpDepartmentRankMapper departmentRankMapper;

    @Override
    public List<HpgpBusyIdlePrediction> busyIdlePrediction(String deptName) {
        // 查询科室排名
        // 组装医院代码、科室代码
        // 查询医院忙闲
        List<HpgpDepartmentRank> deptRank = departmentRankMapper.getRankByStandardDept(deptName, 3);
        if (CollectionUtil.isEmpty(deptRank)) {
            return null;
        }
        Set<String> hospitalCodes = deptRank.stream().map(HpgpDepartmentRank::getHospitalCode).collect(Collectors.toSet());
        Set<String> deptCodes = deptRank.stream().map(HpgpDepartmentRank::getDeptCode).collect(Collectors.toSet());

        return this.list(
                new LambdaQueryWrapper<HpgpBusyIdlePrediction>()
                        .in(HpgpBusyIdlePrediction::getHospitalCode, hospitalCodes)
                        .in(HpgpBusyIdlePrediction::getDeptCode, deptCodes)
                        .gt(HpgpBusyIdlePrediction::getPredictionDate, "")
                        .le(HpgpBusyIdlePrediction::getPredictionDate, "")
        );
    }
}
