package cn.sh.stc.sict.theme.hphy.controller.mp;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.net.Ipv4Util;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.constant.Constant;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.common.core.util.WebUtils;
import cn.sh.stc.sict.cloud.common.security.util.SecurityUtils;
import cn.sh.stc.sict.cloud.upms.dto.CurrentUser;
import cn.sh.stc.sict.theme.hphy.constant.PatientConstant;
import cn.sh.stc.sict.theme.hphy.model.HpAppointment;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HpHosInfo;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientBase;
import cn.sh.stc.sict.theme.hphy.service.HpAppointmentService;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpHosInfoService;
import cn.sh.stc.sict.theme.hphy.service.HphyPatientBaseService;
import cn.sh.stc.sict.theme.hphy.wd.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.netty.util.internal.MacAddressUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import org.springframework.web.bind.annotation.*;

/**
 * @author F_xh
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/c/appointment")
@Api(tags = "[C]预约挂号管理")
public class AppointmentController {

    private HpAppointmentService hpAppointmentService;
    private HpDocInfoService hpDocInfoService;
    private HphyPatientBaseService hphyPatientBaseService;
    private HpHosInfoService hosInfoService;

    @ApiOperation("预约须知")
    @GetMapping("/app/rule")
    public R getAppRule(HosInfo hosInfo) {
        HospitalCfg cfg = WanDaHttpUtil.getHospitalCfg(hosInfo);
        return new R(cfg);
    }

    @ApiOperation("预约")
    @PostMapping("/app")
    public R appointment(@RequestBody OrderInfo order) throws Exception {
        CurrentUser current = SecurityUtils.getCurrentUser();
        if (StrUtil.isBlank(current.getOpenId())) {
            return new R().error("患者信息异常！");
        }
        if (StrUtil.isBlank(order.getHosOrgCode())) {
            return new R().error("预约医院信息不可为空！");
        }
        if (StrUtil.isBlank(order.getDoctorId())) {
            return new R().error("预约医生信息不可为空！");
        }
        if (StrUtil.isBlank(order.getScheduleId()) || StrUtil.isBlank(order.getNumSourceId())) {
            return new R().error("预约排班号源信息不可为空！");
        }
//        HpHosInfo hos = hosInfoService.getById(order.getHosOrgCode());
//        if (null == hos) {
//            return new R().error("未找到医院信息！");
//        }
        HphyPatientBase patient = hphyPatientBaseService.getByOpenId(current.getOpenId());
        if (null == patient) {
            return new R().error("未找到患者信息！");
        }
        HpDocInfo docInfo = hpDocInfoService.getById(order.getDoctorId());
        if (null == docInfo) {
            return new R().error("未找到医生信息！");
        }
        order.setOneDeptCode(docInfo.getOneDeptCode());
        order.setDeptCode(docInfo.getDeptCode());
        order.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
        order.setResourceCode(docInfo.getResourceCode());
        order.setChannelCode(WanDaConstant.CHANNEL_CODE_OTHER);
        order.setPayMode(WanDaConstant.PAY_MODE_HOSP);
        order.setUserCardType(WanDaConstant.USER_CARD_TYPE_ID);
        order.setUserCardId(patient.getCertId());
        order.setUserName(patient.getName());
        order.setUserPhone(patient.getPhone());
        order.setUserSex(PatientConstant.getNomalGender(patient.getGender()));
        order.setReplaceUserCardType(WanDaConstant.USER_CARD_TYPE_ID);
        order.setReplaceUserCardId(patient.getCertId());
        order.setReplaceUserName(patient.getName());
        order.setIpAddr(WebUtils.getIP());
        order.setMacAddr(WanDaConstant.DEFAULT_MAC_ADDR);
        order.setPlatformCode(WanDaConstant.PLATFORM_CODE);
        order.setPlatformType(WanDaConstant.PLATFORM_TYPE);
        order.setChannelName(WanDaConstant.CHANNEL_NAME);
        order.setNumSourceFrom(WanDaConstant.NUM_SOURCE_FROM);
        order.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
        if (StrUtil.isBlank(order.getMediCardId()) || StrUtil.isBlank(order.getMediCardType())){
            order.setMediCardId(patient.getMediCardId());
            order.setMediCardType(patient.getMediCardType());
        }
        order.setUserBD(DateUtil.format(IdcardUtil.getBirthDate(patient.getCertId()), DatePattern.NORM_DATE_PATTERN));
        AppResult result = WanDaHttpUtil.submitOrderByUserInfo(order);
        if (null != result) {
            MapperFactory factory = new DefaultMapperFactory.Builder().build();
            HpAppointment app = factory.getMapperFacade().map(order, HpAppointment.class);
            app.setOrderId(result.getOrderId());
            app.setPatientId(patient.getId().toString());
            app.setDoctorId(docInfo.getId().toString());
            app.setResourceCertId(docInfo.getPersonId());
            app.setNumSourceDetailId(result.getNumSourceDetailId());
            app.setNumPassword(result.getNumPassword());
            app.setRoom(result.getRoom());
            app.setHosNumSourceId(result.getHosNumSourceId());
            app.setStatus(Constant.STRING_NO);
            app.setHosOrgName(docInfo.getHosName());
            app.setDeptName(docInfo.getDeptName());
            app.setOrderNumType(order.getOrderNumType());
            app.setPatientName(patient.getName());
            app.setResourceName(docInfo.getResourceName());
            app.setStatus(Constant.STRING_NO);
            hpAppointmentService.save(app);
            return new R().success(result);
        }
        return new R().error("预约失败！");
    }

    @ApiOperation("取消预约")
    @PostMapping("/cancel/app")
    public R cancelAppointment(@RequestBody CancelOrderInfo order) throws Exception {
        if (StrUtil.isBlank(order.getAppId())) {
            return new R().error("预约信息不可为空！");
        }
        HpAppointment app = hpAppointmentService.getById(order.getAppId());
        if (null == app) {
            return new R().error("未找到预约信息！");
        }
        order.setCancelObj(WanDaConstant.CANCELOBJ_PATIENT);
        order.setHosOrgCode(app.getHosOrgCode());
        order.setOrderId(app.getOrderId());
        boolean flag = WanDaHttpUtil.orderCancelInfo(order);

        if (flag) {
            HpAppointment update = new HpAppointment();
            update.setId(app.getId());
            update.setStatus(Constant.STRING_YES);
            hpAppointmentService.updateById(update);
            return new R();
        }
        return new R().error("取消预约失败！");
    }

    @ApiOperation("预约列表")
    @GetMapping("/app/list")
    public R appList(Page page,
                     @RequestParam(value = "startDate", required = false) String startDate,
                     @RequestParam(value = "endDate", required = false) String endDate) {
        CurrentUser current = SecurityUtils.getCurrentUser();
        if (StrUtil.isBlank(current.getOpenId())) {
            return new R().error("患者信息异常！");
        }
        HphyPatientBase patient = hphyPatientBaseService.getByOpenId(current.getOpenId());
        if (null == patient) {
            return new R().error("未找到患者信息！");
        }
        LambdaQueryWrapper<HpAppointment> wrapper = new LambdaQueryWrapper<>();
        wrapper.ge(StrUtil.isNotBlank(startDate), HpAppointment::getNumSourceDate, startDate)
                .le(StrUtil.isNotBlank(endDate), HpAppointment::getNumSourceDate, endDate)
                .eq(HpAppointment::getPatientId, patient.getId())
                .orderByDesc(HpAppointment::getCreateTime);
        page = hpAppointmentService.page(page, wrapper);

        return new R(page);
    }

    @ApiOperation("预约详情")
    @GetMapping("/app/info")
    public R appInfo(@RequestParam("appId")String appId) {
        HpAppointment app = hpAppointmentService.getById(appId);
        return new R(app);
    }
}
