package cn.sh.stc.sict.theme.hphy.controller.mp;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.constant.Constant;
import cn.sh.stc.sict.cloud.common.core.constant.enums.LoginTypeEnum;
import cn.sh.stc.sict.cloud.common.core.dto.WDUserCardInfo;
import cn.sh.stc.sict.cloud.common.core.dto.WDUserInfo;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.common.core.util.SsbUtil;
import cn.sh.stc.sict.cloud.common.core.util.ValidateCodeUtil;
import cn.sh.stc.sict.cloud.common.security.util.SecurityUtils;
import cn.sh.stc.sict.cloud.upms.dto.CurrentUser;
import cn.sh.stc.sict.cloud.upms.model.SysUserBase;
import cn.sh.stc.sict.theme.hphy.constant.PatientConstant;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientBase;
import cn.sh.stc.sict.theme.hphy.service.HphyPatientBaseService;
import cn.sh.stc.sict.theme.hphy.wd.WanDaHttpUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 患者基本信息
 *
 * @author F_xh。
 * @date 2020-09-29 15:05:29
 */
@RestController
@RequestMapping("/hphy/c/patientbase")
@AllArgsConstructor
@Slf4j
@Api(tags = "[C]患者基本信息")
public class HphyPatientBaseController {

    private HphyPatientBaseService hphyPatientBaseService;
    private StringRedisTemplate redisTemplate;

    /**
     * 注册并关联登录信息
     *
     * @param patient
     * @return
     */
    @ApiOperation("注册")
    @PostMapping("/register")
    public R register(@RequestBody HphyPatientBase patient) {
        CurrentUser current = SecurityUtils.getCurrentUser();
        HphyPatientBase base = hphyPatientBaseService.getByOpenId(current.getOpenId());
        patient.setOpenId(current.getOpenId());

        if (base != null && NumberUtil.isNotNullOrZero(base.getId())) {
            patient.setId(base.getId());
        } else {
            patient.setPhone(current.getPhone());
        }
        // 身份证验证
        if (StrUtil.isBlank(patient.getCertId())) {
            return new R().error("身份证号不可为空！");
        }
        if (!IdcardUtil.isValidCard(patient.getCertId())) {
            return new R().error("身份证号验证失败！");
        }
        // 获取性别，年龄
        patient.setAge(String.valueOf(IdcardUtil.getAgeByIdCard(patient.getCertId())));
        patient.setGender(IdcardUtil.getGenderByIdCard(patient.getCertId()) == 1 ? "男" : "女");
        // 手机号redis验证
        // 根据手机号去重
        // 获取家庭医生签约信息
        try {
            String familyDocHos = WanDaHttpUtil.getContractInfo(patient.getCertId());
            if (StrUtil.isNotBlank(familyDocHos)) {
                patient.setFamilyDocHos(familyDocHos);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        patient.setVerifyStatus(Constant.STRING_YES);
        hphyPatientBaseService.saveOrUpdate(patient);

        return new R(patient.getId());
    }

    /**
     * 1. 发送注册短信验证码[调用黄埔webservice接口]
     */

    /**
     * F_xh
     * 获取患者信息
     *
     * @return
     */
    @GetMapping
    public R getInfo() {
        CurrentUser current = SecurityUtils.getCurrentUser();

        if (StrUtil.isBlank(current.getOpenId())) {
            log.error("openId 为空");
        }
        // 获取患者信息
        HphyPatientBase base = hphyPatientBaseService.getByOpenId(current.getOpenId());

        if (base != null && StrUtil.isNotBlank(base.getGender())) {
            current.setGender(PatientConstant.getGender(base.getGender()));
            SecurityUtils.updateCurrent(current);
        }

        return new R(base);
    }

    @ApiOperation("第三方公众号进入系统时，获取患者信息")
    @GetMapping("/third-party")
    public R getInfoByToken(@RequestParam("source") String source,
                            @RequestParam("token") String token) {
        CurrentUser current = SecurityUtils.getCurrentUser();

        if (StrUtil.isBlank(current.getOpenId())) {
            log.error("openId 为空");
        }
        // 获取患者信息
        HphyPatientBase base = hphyPatientBaseService.getByOpenId(current.getOpenId());
        if( ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())){
            if(LoginTypeEnum.SSB.getType().equals(source)){
                WDUserInfo userInfo = SsbUtil.getUserInfo(token);
                List<WDUserCardInfo> cardList = SsbUtil.getUserCardInfoList(token);
                base = hphyPatientBaseService.saveSSbInfo(current, userInfo, cardList);
            }
        }

        if( ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())){
            return new R().error("患者未实名！");
        }
        if (StrUtil.isNotBlank(base.getGender())) {
            current.setGender(PatientConstant.getGender(base.getGender()));
            SecurityUtils.updateCurrent(current);
        }

        return new R(base);
    }
}
