package cn.sh.stc.sict.theme.hphy.service.impl;

import cn.sh.stc.sict.theme.hphy.constant.DataConstant;
import cn.sh.stc.sict.theme.hphy.dao.HpDeptInfoMapper;
import cn.sh.stc.sict.theme.hphy.dao.HpDocInfoMapper;
import cn.sh.stc.sict.theme.hphy.dao.HpHosInfoMapper;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HpHosInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDeptInfoService;
import cn.sh.stc.sict.theme.hphy.vo.DeptDoctorsVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * (HpDeptInfo)表服务实现类
 *
 * @author makejava
 * @since 2021-01-28 10:35:58
 */
@Slf4j
@AllArgsConstructor
@Service("hpDeptInfoService")
public class HpDeptInfoServiceImpl extends ServiceImpl<HpDeptInfoMapper, HpDeptInfo> implements HpDeptInfoService {
    private final HpDocInfoMapper hpDocInfoMapper;
    private final HpHosInfoMapper hpHosInfoMapper;

    @Override
    public DeptDoctorsVO getDeptDoctors(String hospitalCode, String deptCode) {
        HpHosInfo hosInfo = hpHosInfoMapper.selectById(hospitalCode);
        // 查询科室信息
        // 查询医生信息
        HpDeptInfo deptInfo = this.getOne(
                new LambdaQueryWrapper<HpDeptInfo>()
                        .eq(HpDeptInfo::getHosOrgCode, hospitalCode)
                        .and(i -> i.eq(HpDeptInfo::getOneDeptCode, deptCode)
                                        .or().eq(HpDeptInfo::getDeptCode, deptCode))
                        .orderByDesc(HpDeptInfo::getDeptLevel)
        );
        if (deptInfo == null){
            log.error("根据医院编码：{} 和科室编码：{} 未查出科室信息", hospitalCode, deptCode);
            return null;
        }

        LambdaQueryWrapper<HpDocInfo> docWrapper = new LambdaQueryWrapper<>();
        docWrapper.eq(HpDocInfo::getHosOrgCode, hospitalCode);
        docWrapper.eq(HpDocInfo::getOneDeptCode, deptInfo.getOneDeptCode());
        if (DataConstant.TWO_DEPT.equals(deptInfo.getDeptLevel())){
            docWrapper.eq(HpDocInfo::getDeptCode, deptInfo.getDeptCode());
        }
        docWrapper.orderByAsc(HpDocInfo::getIndexNo);

        List<HpDocInfo> doctors = hpDocInfoMapper.selectList(docWrapper);

        return new DeptDoctorsVO(hosInfo, deptInfo, doctors);
    }
}
