package cn.sh.stc.sict.theme.hphy.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.sh.stc.sict.theme.hphy.dao.HpDocInfoMapper;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HphyGuideMatch;
import cn.sh.stc.sict.theme.hphy.model.HphyGuideRecord;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import cn.sh.stc.sict.theme.hphy.service.HphyGuideMatchService;
import cn.sh.stc.sict.theme.hphy.vo.DoctorMatchVO;
import cn.sh.stc.sict.theme.hphy.vo.PythonResult;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * (HpDocInfo)表服务实现类
 *
 * @author makejava
 * @since 2021-01-28 10:35:59
 */
@Service("hpDocInfoService")
@AllArgsConstructor
public class HpDocInfoServiceImpl extends ServiceImpl<HpDocInfoMapper, HpDocInfo> implements HpDocInfoService {

    private HphyGuideMatchService hphyGuideMatchService;

    @Override
    public boolean doctorExist(String resourceCode) {
        return this.getById(resourceCode) != null;
    }

    @Override
    public List<DoctorMatchVO> getRecommendList(HphyGuideRecord guide, List<PythonResult> pythonList) {
        List<DoctorMatchVO> result = new ArrayList<>();
        if(CollUtil.isNotEmpty(pythonList)){
            Map<String, Double> matchMap = pythonList.stream().collect(Collectors.toMap(PythonResult::getDoctorId, PythonResult::getMatchScore));
            List<String> doctorIdList = pythonList.stream().map(PythonResult::getDoctorId).collect(Collectors.toList());
            LambdaQueryWrapper<HpDocInfo> wrapper = new LambdaQueryWrapper<>();
            wrapper.in(HpDocInfo::getPersonId, doctorIdList);
            List<HpDocInfo> list = this.list(wrapper);
            List<HphyGuideMatch> matchList = new ArrayList<>();
            list.forEach(doctor -> {
                DoctorMatchVO vo = new DoctorMatchVO();
                vo.setDocInfo(doctor);
                vo.setMatchScore(matchMap.getOrDefault(doctor.getPersonId(), 85.0));
                result.add(vo);
                HphyGuideMatch match = new HphyGuideMatch();
                match.setDeptId(doctor.getDeptCode());
                match.setDeptName(doctor.getDeptName());
                match.setDoctorId(doctor.getResourceCode());
                match.setDoctorName(doctor.getResourceName());
                match.setGuideId(guide.getId());
                match.setHospId(doctor.getHosOrgCode());
                match.setHospName(doctor.getHosName());
                match.setMatchScore(vo.getMatchScore().toString());
                match.setPatientId(guide.getPatientId());
                match.setPatientName(guide.getPatientName());
                matchList.add(match);
            });
            // 保存医生推荐记录
            hphyGuideMatchService.asyncSaveBatch(matchList);
        }

        return result.stream()
                .sorted(Comparator.comparing(DoctorMatchVO::getMatchScore).reversed())
                .collect(Collectors.toList());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveDoctor(List<HpDocInfo> docInfos, String hosOrgCode) {
        this.remove(new LambdaQueryWrapper<HpDocInfo>().eq(HpDocInfo::getHosOrgCode, hosOrgCode));
        this.saveBatch(docInfos);
    }
}